%Made by J.T.B. Overvelde on 9 may 2011

clc; close all; clear all; format compact

global rho mu g dt nX nY nXp nYp w h boundU boundV boundP N hx hy

%initialize variables and grid
initVar;
[x, y, xp, yp, u, v, p] = gridInit;

%1 = Explicit Euler
%2 = Adam Bashford 2
firstStep = 1;
solver = 2;
nMaxCycles = 5001;
nSave = 100;

% boundary conditions velocity and pressure
% [Side type value]
% Side: 1 right
%       2 left
%       3 bottom
%       4 top
% Type: 1 Constant
%       2 Constant derivative
% Value: Velocity value or Pressure value

Uin=0.001
boundU =    [1 2 0;
             2 1 Uin;
             3 1 0;
             4 1 0];
boundV =    [1 1 0;
             2 1 0;
             3 1 0;
             4 1 0];
boundP =    [1 1 0;
             2 2 0;
             3 2 0;
             4 2 0]; 

%initial condition
u(:,:) = Uin; 

%initialize matrices
[A,f]=PresProjInit();

t=0; dtStart=dt;nCycles = 1; firstIterate=1;
while nCycles<nMaxCycles
    nCycles
    % first, calculate uStar, vStar
    if solver == 1
        % explicit euler
        [uStar,vStar,fu,fv] = explicitEuler(u,v);
    elseif solver == 2
        % adam bashford2
        if firstIterate==1
            [uStar,vStar,fu1,fv1] = explicitEuler(u,v);
            firstIterate = firstIterate + 1;
        else
            [uStar,vStar,fu1,fv1] = adamBashford2(u,v,fu1,fv1);
        end
    end
    
    % boundary conditions for uStar and vStar
    for i=1:4
        uStar = boundaryConditions(uStar,boundU(i,1),boundU(i,2),boundU(i,3));
        vStar = boundaryConditions(vStar,boundV(i,1),boundV(i,2),boundV(i,3));
    end
    
    % apply pressure correction
    [u,v,p] = PresProj(x,y,uStar,vStar,A,f);

    %calculate max residu continuity
    res=zeros(nX,nY);
    %res(2:nX-1,2:nY-1)=(u(3:nX,2:nY-1)-u(1:nX-2,2:nY-1))/(2*hx)+(v(2:nX-1,3:nY)-v(2:nX-1,1:nY-2))/(2*hy);
    for i=2:nXp-1
        for j=2:nYp-1
            res(i,j)=1/2*(u(i,j)-u(i-1,j)+u(i,j-1)-u(i-1,j-1))/hx+...
            1/2*(v(i,j)-v(i,j-1)+v(i-1,j)-v(i-1,j-1))/hy;
        end
    end
    
    %plot every nSave cycles
    if mod(nCycles,nSave)==0
       figure(2)
       render(u,v,p,res,x,y,xp,yp,t,dt,nCycles);
       pause(0.2);
    end    
    
    t=t+dt;
    nCycles = nCycles+1;
end
